#!/usr/local/bin/perl

#
# Bid Monkey v1.2.1 (does your ebay bidding for you)
#
# pedram amini <pedram@redhive.com> <http://pedram.redhive.com>
#
# synopsis:
#   - connect to internet if we have to.
#   - determine time left on auction.
#   - disconnect from internet.
#   - sleep for most of that time.
#   - reconnect to internet.
#   - re-determine time left on auction.
#   - "review bid".
#   - sleep until the last minute.
#   - place the actual bid.
#
# why do i determine the time left on the auction twice? increased accuracy.
#
# contributions:
#   mark kahrs
#       - XHTML regex updates.
#
#   stefan luethje:
#       - interactive input.
#       - more information for output.
#       - chdir(), so you can execute script without cd-ing to it.
#
#   dietrich rothe
#       - fixed the currency extraction regex (for international).
#       - original addition of get_online / get_offline calls.
#       - measuring of monkey_review_bid delay for increased accuracy.
#
# version 1.2.1 changes
#   - updated a regex to work with ebays new XHTML compatible code.
#
# version 1.2 changes
#   - added all contributions from dietrich rothe.
#   - note: the get[online,offline] scripts are really a unix only thing.
#   - made the necessary changes to input/output interfaces to accomodate for
#     those additions.
#
# version 1.1 changes
#   - fixed problem associated with ebay changing their directory structure.
#   - added code to handle moved objects.
#   - added all contributions from stefan luethje.
#   - some other minor changes i can't remember.

use IO::Socket;
use Getopt::Long;
use File::Basename;
use strict;

chdir(dirname($0));

# monkey ask interactive flags.
my $F_VALUE_INT = 1;

# support files
require 'monkey_functions.pl';

# globals.
my @USER_AGENTS = ();
my $UA_DB       = "db/user_agents.db";

# locals.
my $help              = 0;
my $interactive       = 0;
my $item_num          = 0;
my $max_bid           = 0;
my $quantity          = 1;
my $username          = "";
my $password          = "";
my $safety            = 120;
my $price             = 0;
my $currency          = "";
my $dialup            = 0;
my $dialup_time       = 60;
my $time_before_delay = 0;

my ($article, $currency, $daemonize, $days, $ebay_key, $headers, $hours, $mins, $price, $secs, $sleep_time, $ua);

#
# parse command line arguments and initialize.
#

print "\n[ bid monkey v1.2 ]\n\n";

GetOptions (
	'daemonize|d'     => \$daemonize,
	'help|h'          => \$help,
	'ask|a'           => \$interactive,
	'item_num|i=i'    => \$item_num,
	'max_bid|b=s'	  => \$max_bid,
	'quantity|q=i'    => \$quantity,
	'username|u=s'	  => \$username,
	'password|p=s'	  => \$password,
	'safety|s=i'      => \$safety,
	'dialup|l'        => \$dialup,
	'dialup_time|t=i' => \$dialup_time
) || monkey_usage();

# interactive input if item_num and max_bid are not set.
if ($interactive && !$help) {
	my $defusername = $username;
	$username    = monkey_ask_interactive("ebay user name", $username);
	$password    = monkey_ask_interactive("ebay password") if ($password eq "" || $defusername ne $username);
	$item_num    = monkey_ask_interactive("ebay item number", $item_num > 0 ? $item_num : undef, $F_VALUE_INT);
	$max_bid     = monkey_ask_interactive("maximum bid amount", $max_bid > 0 ? $max_bid : undef);
	$safety      = monkey_ask_interactive("safety time in seconds", $safety, $F_VALUE_INT);
	$dialup      = monkey_ask_interactive("call get_online / get_offline scripts to connect/disconnect from internet (0 for no, or 1 for yes)", $dialup, $F_VALUE_INT);
	$dialup_time = monkey_ask_interactive("time it takes to connect to internet in seconds", $dialup_time, $F_VALUE_INT) if ($dialup);
	print "\n";
}

# we need a connection.
system("./get_online.sh") if ($dialup);

# parse the maximum bid value.
$max_bid = monkey_parse_max_bid($max_bid);

# make sure all the required arguments are set.
monkey_usage() if ($item_num == 0 || $max_bid == 0 || $username eq "" || $password eq "");


if ($daemonize) {                   # daemonize if we have to.
    print "daemonizing...\n";       # announce what we're about to do.
    fork && exit;                   # spawn a child and kill the parent.
    close STDOUT;                   # close our inherited file descriptors.
    close STDIN;                    # we keep STDERR open for error messages.
}

# initialize database.
monkey_initialize_db($UA_DB);

# retrieve headers we're going to use.
($ua, $headers) = monkey_get_headers();

print "monkey has decided to spoof the following user agent: $ua\n";

# see how much time is left on the auction.
($days, $hours, $mins, $secs, $currency, $price, $article) = monkey_determine_auction_end($item_num, $headers);

# some sanity checking.
monkey_error("couldn't fetch price information on item number $item_num.") if (!$price);

# print the summary.
print "\nmonkey bid summary:";
print "\n    item number:   $item_num";
print "\n    article:       $article";
print "\n    maximum bid:   $currency $max_bid";
print "\n    current price: $currency $price";
print "\n    username:      $username";
print "\n    password:      $password";
print "\n    dynamically connecting to internet." if ($dialup);
print "\n    time it takes to connect to internet in seconds: $dialup_time." if ($dialup);
print "\n    monkey will place bid at $safety seconds before the end of auction.";
print "\n\n";

# make sure the bid is high enough.
monkey_error("your maximum bid is too low.") if ($price > $max_bid);

print "auction ends in: $days days $hours hours $mins minutes $secs seconds.\n";

# we shall sleep until the final hour.
# then awaken to redetermine the remaining time on the auction.
if ($hours > 0) {
    # sleep until the final hour then recalculate auction end time.
    $sleep_time = (3600 * $hours) + (86400 * $days);
    
    # if we need to dialup take that into effect.
    $sleep_time -= $dialup_time if ($dialup);

    # sleep.
    print "monkey is going to sleep for: $days days $hours hours.\n";
    # we won't need the connection for a while.
    system("./get_offline.sh") if ($dialup);
    sleep($sleep_time);

    # we need a connection.
    # we no longer drop the connection after this point to get better accuracy.
    system("./get_online.sh") if ($dialup);
    
    # the final hour has arrived. let us redetermine when the auction ends.
    ($days, $hours, $mins, $secs, $currency, $price) = monkey_determine_auction_end($item_num, $headers);

    print "auction ends in: $days days $hours hours $mins minutes $secs seconds.\n";
}

# store the current time to measure the following procedural delay.
$time_before_delay = time;

# calculate sleep time. (note: hours and days should be zero here).
$sleep_time = $secs + (60 * $mins) + (3600 * $hours) + (86400 * $days) - $safety;

# "review bid" - start bidding process and retrieve ebay key for placing bid.
$ebay_key = monkey_review_bid($item_num, $max_bid, $headers);

# if we don't have the key at this point we need to bail.
monkey_error("key not found") if (!$ebay_key);

# sleep for proper amount of time.
# take into account the procedural delay.
# we count the procedural delay twice, once for monkey_review_bid and once more
# for monkey_place_bid.
print "monkey is going to sleep for: $days days $hours hours $mins minutes $secs seconds - $safety seconds\n";
sleep ($sleep_time - 2 * (time - $time_before_delay));

# place bid.
monkey_place_bid($item_num, $max_bid, $quantity, $username, $password, $ebay_key, $headers);

print "\nbid placed. check your email for the outcome. give monkey banana.\n";

# we're finished with the connection now.
# modify get_offline.sh to timeout if you get annoyed with bid monkey kicking 
# you offline while your working.
system("./get_offline.sh") if ($dialup);
